<?php

namespace App\Services\ThirdPlatform\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class ZhenkhApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('zkh');
        $this->getPlatformInfo();
    }

    /**
     * 获取震坤行平台token
     * @return bool
     * @throws \Exception
     */
    public function getToken()
    {
        try{
            if($this->checkPlatformStatus()){
                $tokenParams = [
                    'accessId' => $this->platformInfo['platform_params']['prod_accessId'],
                    'accessSecret' => $this->platformInfo['platform_params']['prod_accessSecret']
                ];
                if(env('API_DEBUG')){
                    $tokenParams = [
                        'accessId' => $this->platformInfo['platform_params']['test_accessId'],
                        'accessSecret' => $this->platformInfo['platform_params']['test_accessSecret']
                    ];
                }
                $response = $this->getPostClient($this->getPlatformApiUrl().'/openApi/v1/token',$tokenParams);
                $token = $this->apiResponse($response);

                return $token['data'];
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * 获取震坤行 供应商商品数据
     * @param $pageNo
     * @param $pageSize
     * @param $token
     */
    public function getUpdateGoods($pageNo,$pageSize,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->getPlatformApiUrl().'/openApi/v1/common/goodPageList?pageNo='.$pageNo.'&pageSize='.$pageSize,
                    [
                        'headers' => [
                            'honeycombToken' => $token,
                            'Content-Type' => 'application/json;charset=UTF-8'
                        ]
                    ]
                );

                $goodsList = $this->apiResponse($response);
                if($goodsList){
                    return [
                        'list' => $goodsList['data'],
                        'total_page' => $goodsList['totalPage']
                    ];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    public function getUpdateOrders($params,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->getPostClient(
                    $this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder',
                    [
                        'updateTimeBegin' => $params['start_time'],
                        'updateTimeEnd' => $params['end_time'],
                        'pageSize' => $params['page_size'],
                        'pageNo' => $params['page_no'],
                    ],null,
                    [
                        'honeycombToken' => $token,
                        'Content-Type' => 'application/json;charset=UTF-8'
                    ]
                );

                $orderList = $this->apiResponse($response);
                if($orderList){
                    return [
                        'list' => $orderList['data'],
                        'total_page' => $orderList['data']['totalPages']
                    ];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @获取订单详情
     * @param $orderId
     * @param $token
     * @return mixed|void
     */
    public function getUpdateOrdersDetail($orderId,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder/detail?purchaseOrderId='.$orderId,
                    [
                        'headers' => [
                            'honeycombToken' => $token,
                            'Content-Type' => 'application/json;charset=UTF-8'
                        ]
                    ]
                );

                $orderDetail = $this->apiResponse($response);
                if($orderDetail){
                    return $orderDetail['data'];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @获取物流信息
     * @param $token
     * @return mixed|void
     */
    public function expressLogistics($token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->getPlatformApiUrl().'/openSupplierApi/v1/expressLogistics',
                    [
                        'headers' => [
                            'honeycombToken' => $token,
                            'Content-Type' => 'application/json;charset=UTF-8'
                        ]
                    ]
                );

                $orderDetail = $this->apiResponse($response);
                if($orderDetail){
                    return $orderDetail['data'];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    public function sendDeliveryOrder($orderId,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder/getDeliveryOrderPart?purchaseOrderId='.$orderId,
                    [
                        'headers' => [
                            'honeycombToken' => $token,
                            'Content-Type' => 'application/json;charset=UTF-8'
                        ]
                    ]
                );

                $deliveryOrderDetail = $this->apiResponse($response);
                if($deliveryOrderDetail){
                    return $deliveryOrderDetail['data'];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @发货确认
     */
    public function ackDeliveryOrder($deliveryOrder, $token)
    {
        try{
            if($this->checkPlatformStatus()){
               $response = $this->getPostClient($this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder/ackDeliveryOrderPart',
                    $deliveryOrder,null,
                    [
                        'honeycombToken' => $token,
                        'Content-Type' => 'application/json;charset=UTF-8'
                    ]);

                return $this->apiResponse($response);
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @param $params
     * @return bool|void
     * @throws \Exception
     */
    public function updateReportProductPrice($params, $token)
    {
        if(empty($params)){
            return false;
        }
        try{
            if($this->checkPlatformStatus()){
                $response = $this->getPostClient($this->getPlatformApiUrl().'/openApi/v1/common/report/price',
                    [[
                        'customPrice' => $params['price'],
                        'priceUnit' => $params['price_unit'],
                        'zkhSku' => $params['sku']
                    ]],null,
                    [
                        'honeycombToken' => $token,
                        'Content-Type' => 'application/json;charset=UTF-8'
                    ]);

                return $this->apiResponse($response);
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @上报商品库存
     * @param $params
     * @param $token
     * @return bool|mixed|void
     */
    public function updateReportProductStock($params, $token)
    {
        if(empty($params)){
            return false;
        }
        try{
            if($this->checkPlatformStatus()){
                $response = $this->getPostClient($this->getPlatformApiUrl().'/openApi/v2/common/report/stock',
                    [[
                        'supplierWarehouseName' => $params['warehouse'],
                        'warehouseNameList' => $params['address'],
                        'qty' => intval($params['qty']),
                        'zkhSku' => $params['sku']
                    ]],null,
                    [
                        'honeycombToken' => $token,
                        'Content-Type' => 'application/json;charset=UTF-8'
                    ]);

                return $this->apiResponse($response);
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @获取送货单pdf
     * @param $deliveryCode
     * @param $token
     * @return mixed|void
     */
    public function getDeliveryOrderDetailForPDF($deliveryCode, $token)
    {
        try{
            if($this->checkPlatformStatus()){
                $response = $this->clientRequest('get',
                    $this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder/getDeliveryOrderDetailForPDF?deliveryCode='.$deliveryCode,
                    [
                        'headers' => [
                            'honeycombToken' => $token,
                            'Content-Type' => 'application/json;charset=UTF-8'
                        ]
                    ]
                );

                $delivery = $this->apiResponse($response);
                if($delivery){
                    return $delivery['data'];
                }
            }

        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    /**
     * @订单确认
     * @param $orderNo
     * @param $token
     * @return bool|mixed|void
     */
    public function orderConfirm($orderNo, $token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['purchaseOrderId'] = $orderNo;
                $response = $this->getPostClient($this->getPlatformApiUrl().'/openPoApi/v1/purchaseOrder/confirm',
                    $params,null,
                    [
                        'honeycombToken' => $token,
                        'Content-Type' => 'application/json;charset=UTF-8'
                    ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $exception){
            $this->requestError($exception);
        }
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        if($decodeResponse){
            if($decodeResponse['code'] == '200' && $decodeResponse['isSuccess'] == true){
                return $decodeResponse['data'] == NULL ? true : $decodeResponse;
            }elseif($decodeResponse['code'] == '403' && $decodeResponse['msg'] == 'token 已经失效'){
                //删除当前token
                $this->removePlatformDataEntries('zkh_api_token');
                SimpleLogs::writeLog('token 已经失效',__CLASS__.':apiResponse');
                return ['reset' => true];
            }
        }
        return false;
    }

    protected function requestError($response)
    {
        SimpleLogs::writeLog($response->getMessage(), 'Zhenkh api error', 'error');
        throw $response;
    }

    private function getPlatformApiUrl()
    {
        if(env('API_DEBUG')){
            return $this->platformInfo['platform_params']['test_platform_url'];
        }
        return $this->platformInfo['platform_url'];
    }
}
